#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
2025-03-11 09:23:21 


Asterisk AGI Sprechende Uhr mit Zeitschleife
Ansagen alle 15 Sekunden (0, 15, 30, 45), Beep vor Text, Zeit 15 Sekunden vorgeschoben
Eigenständiges Skript ohne externe Abhängigkeiten außer gTTS und ffmpeg
Volker Lange-Janson, 9. März 2025
asterisk@janson-soft.de
Frei verfügbar für jeden auf meiner Webseite – macht damit, was ihr wollt!

Entwicklung:
https://grok.com/share/bGVnYWN5_d0bab6c9-1945-451a-b79d-586c3a94fc4c
https://elektronikbasteln.pl7.de/telefonische-zeitansage-fuer-asterisk-als-python-agi-skript-mit-grok-3-und-claude-entwickelt

Damit das Skript läuft:
https://grok.com/share/bGVnYWN5_0ed46786-78eb-4b69-a7a9-7661a151842d

Verbesserungen vom 11.3.25 9 Uhr 21:
1. Zur Sicherheit und zur besseren Kontrolle werden Die WAV-Dateien erst 2 Sekunden nach dem Abspielen gelöscht.

2. Die WAV-Datei wird erst 5 Sekunden nach dem Signalton abgespielt. Das klingt angenehmer.

3. Die WAV-Dateien werden in einem sprachspezifischen Ordner abgelegt, um Klarheit zu schaffen und gegenseitige Beeinflussungen zu vermeiden. Für die deutsche Ansage gilt der z.B. "/usr/share/asterisk/sounds/test/de/".

4. Wenn dieselbe Nummer gleichzeitig von meheren Teilnehmern angerufen wir, werden die WAV-Dateien nicht korrekt abgespielt, weil offensichtlich eine WAV-Datei nicht von mehreren Endgerät gleichzeitig abgespielt werden kann. Der Fall kommt zwar in meinem Fall selten vor, ist aber ein Fehler. Abhilfe: Eindeutige Dateinamen mit UUID (für mehrere Anrufer). In play_text_as_audio wurde eine eindeutige ID mit uuid hinzugefügt (unique_id = str(uuid.uuid4())[:8]), sodass jede Ansage einen individuellen Dateinamen bekommt, z.B. time_12_0_15_abc123de.wav statt nur time_12_0_15.wav. Warum? Damit mehrere Anrufer gleichzeitig ihre eigene WAV-Datei haben und es keine Überschreibungen oder Konflikte gibt. Das war der Schlüssel, um das Problem mit parallelen Anrufen zu lösen. Jeder Anrufer bekommt seine eigene Datei, und alle 5 Testanrufe liefen problemlos.

"""

import sys
import datetime
import time
import logging
import os
import uuid  # Für eindeutige Dateinamen
from gtts import gTTS

# Logging konfigurieren
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(message)s')
logger = logging.getLogger(__name__)

# Konstanten
INTERVALS = [0, 15, 30, 45]  # Ansagen bei diesen Sekunden
TIME_SHIFT = 15  # Sekunden, um die die angekündigte Zeit vorgeschoben wird
DELETE_DELAY = 2  # Sekunden Verzögerung vor dem Löschen der WAV-Datei
WAIT_AFTER_BEEP = 5  # Warten mit der Ansage nach dem Signalton

# AGI-spezifische Funktionen
def init_env():
    """Initialisiert die AGI-Umgebung und gibt sie als Dictionary zurück."""
    env = {}
    while True:
        line = sys.stdin.readline().strip()
        if line == '':
            break
        key, data = line.split(':', 1)
        if key[:4] != 'agi_':
            sys.stderr.write("Did not work!\n")
            sys.stderr.flush()
            continue
        key = key.strip()
        data = data.strip()
        if key != '':
            env[key] = data

    sys.stderr.write("AGI Environment Dump:\n")
    sys.stderr.flush()
    for key in env.keys():
        sys.stderr.write(f" -- {key} = {env[key]}\n")
        sys.stderr.flush()
    
    return env

def sayit(params):
    """Spielt eine Audiodatei über Asterisk ab."""
    sys.stderr.write(f"STREAM FILE {str(params)} \"\"\n")
    sys.stderr.flush()
    sys.stdout.write(f"STREAM FILE {str(params)} \"\"\n")
    sys.stdout.flush()
    result = sys.stdin.readline().strip()
    return result

def text_to_speech(text, output_file="output"):
    """Konvertiert Text in Sprache und speichert ihn als WAV-Datei."""
    try:
        output_file = f"/usr/share/asterisk/sounds/test/de/{output_file}"
        temp_mp3 = f"{output_file}.mp3"
        temp_wav = f"{output_file}.wav"
        tts = gTTS(text=text, lang="de")

        tts.save(temp_mp3)
        os.system(f"ffmpeg -i {temp_mp3} -ar 8000 -ac 1 -y {temp_wav}")
        os.remove(temp_mp3)

        sys.stderr.write(f"Text-to-Speech file saved as {temp_wav}\n")
        sys.stderr.flush()
        return temp_wav
    except Exception as e:
        sys.stderr.write(f"Error during text-to-speech conversion: {e}\n")
        sys.stderr.flush()
        return None

def play_text_as_audio(text, output_file="output"):
    """Konvertiert Text in Sprache, streamt ihn an Asterisk und löscht die WAV-Datei nach Verzögerung."""
    # Eindeutiger Dateiname mit UUID
    unique_id = str(uuid.uuid4())[:8]  # Kurze eindeutige ID
    unique_output_file = f"{output_file}_{unique_id}"
    wav_file = text_to_speech(text, unique_output_file)
    if wav_file:
        time.sleep(WAIT_AFTER_BEEP)  # Warten nach dem Signalton
        sayit(wav_file.replace(".wav", ""))  # Asterisk erwartet Pfad ohne Erweiterung
        time.sleep(DELETE_DELAY)  # 2 Sekunden warten vor dem Löschen
        os.remove(wav_file)  # WAV-Datei nach Verzögerung löschen
        sys.stderr.write(f"Gelöschte WAV-Datei nach {DELETE_DELAY}s Verzögerung: {wav_file}\n")
        sys.stderr.flush()

# AGI-Umgebung initialisieren
agi_env = init_env()

# Uhr-spezifische Funktionen
def generate_time_str(tone_time):
    """Erzeugt den Text für die Zeitansage mit vorgeschobener Zeit."""
    shifted_time = tone_time + datetime.timedelta(seconds=TIME_SHIFT)
    
    hour = shifted_time.hour
    minute = shifted_time.minute
    second = shifted_time.second
    
    second_str = "1 Sekunde" if second == 1 else f"{second} Sekunden"
    minute_str = "1 Minute" if minute == 1 else f"{minute} Minuten"
    
    return f"Beim nächsten Ton ist es {hour} Uhr {minute_str} und {second_str}"

def play_tone():
    """Spielt den Asterisk-internen Beep-Ton ab."""
    sayit("beep")

def get_next_announcement_time():
    """Berechnet die nächste Ansagezeit im 15-Sekunden-Takt."""
    now = datetime.datetime.now()
    current_second = now.second
    current_minute = now.replace(second=0, microsecond=0)
    
    for interval in INTERVALS:
        if interval > current_second or (interval == 0 and current_second > 45):
            next_time = current_minute.replace(second=interval)
            if interval == 0 and current_second > 45:
                next_time += datetime.timedelta(minutes=1)
            return next_time
    return current_minute + datetime.timedelta(minutes=1)

def announce_time(tone_time):
    """Spielt zuerst den Ton ab, dann die vorgeschobene Zeitansage."""
    play_tone()
    time_str = generate_time_str(tone_time)
    logger.info(f"Ansage: {time_str}")
    play_text_as_audio(time_str, output_file=f"time_{(tone_time + datetime.timedelta(seconds=TIME_SHIFT)).hour}_{(tone_time + datetime.timedelta(seconds=TIME_SHIFT)).minute}_{(tone_time + datetime.timedelta(seconds=TIME_SHIFT)).second}")

def main():
    """Hauptfunktion mit Zeitschleife für das AGI-Skript."""
    try:
        logger.info("AGI Sprechende Uhr mit Zeitschleife gestartet (Beep vor Text, Zeit +15s)")
        sys.stderr.write("AGI Sprechende Uhr mit Zeitschleife gestartet (Beep vor Text, Zeit +15s)\n")
        sys.stderr.flush()

        while True:
            next_time = get_next_announcement_time()
            now = datetime.datetime.now()
            wait_seconds = (next_time - now).total_seconds()

            if wait_seconds > 0:
                logger.info(f"Warte {wait_seconds:.2f} Sekunden bis {next_time.strftime('%H:%M:%S')}")
                time.sleep(wait_seconds)

            announce_time(next_time)
            time.sleep(0.5)

    except KeyboardInterrupt:
        logger.info("Programm durch Benutzer beendet")
        sys.stderr.write("Programm durch Benutzer beendet\n")
        sys.stderr.flush()
    except Exception as e:
        logger.error(f"Fehler im AGI-Skript: {e}")
        sys.stderr.write(f"Fehler im AGI-Skript: {e}\n")
        sys.stderr.flush()
        sys.exit(1)

if __name__ == "__main__":
    main()
